#!/usr/bin/python

# Copyright (C) 2000, 2001 Progeny Linux Systems, Inc.
# Authors: Branden Robinson, Jeff Licquia
#
# This is free software; you may redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# This is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the Debian GNU/Linux system; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111-1307 USA

# this is my first Python program, so please use lube

import configlet
from gtk import *
from gnome.ui import *
import re
import string
import os

current_horiz_sync = ""
current_vert_refresh = ""
current_video_ram = ""

color_depths_keys = [ "monochrome (1 bit)",
                      "16 colors (4 bits)",
                      "256 colors (8 bits)",
                      "32,768 colors (15 bits)",
                      "65,536 colors (16 bits)",
                      "16.7 million colors (24 bits)" ]

color_depths_values = [ "1", "4", "8", "15", "16", "24" ]

color_depths = { }
for i in range(len(color_depths_keys)):
	color_depths[color_depths_keys[i]] = color_depths_values[i]

bpp_choices = [ "8",
                "16",
                "24",
                "32" ]

card_drivers_keys = [ "Alliance",
                      "Ark Logic",
                      "ATI",
                      "Chips and Technologies",
                      "Cirrus Logic",
                      "Cyrix",
                      "Linux kernel framebuffer driver",
                      "3Dfx Voodoo Graphics / Voodoo2",
                      "3DLabs",
                      "Number Nine Imagine 128",
                      "Intel i740",
                      "Intel i810",
                      "IMS (Integrated Micro Solutions)",
                      "Matrox",
                      "NeoMagic MagicGraph",
                      "NVidia",
                      "Rendition",
                      "S3 ViRGE & Trio3D",
                      "S3 Savage",
                      "SMI (Silicon Motion, Inc.)",
                      "SiS (Silicon Integrated Systems)",
                      "3Dfx Voodoo Banshee / Voodoo3 / Voodoo4 / Voodoo5",
                      "DEC (Digital Equipment Corp.) TGA",
                      "Trident",
                      "Tseng Labs",
                      "generic VESA driver",
                      "generic VGA driver" ]

card_drivers_values = [ "apm",
                        "ark",
                        "ati",
                        "chips",
                        "cirrus",
                        "cyrix",
                        "fbdev",
                        "glide",
                        "glint",
                        "i128",
                        "i740",
                        "i810",
                        "imstt",
                        "mga",
                        "neomagic",
                        "nv",
                        "rendition",
                        "s3virge",
                        "savage",
                        "siliconmotion",
                        "sis",
                        "tdfx",
                        "tga",
                        "trident",
                        "tseng",
                        "vesa",
                        "vga" ]

card_drivers = { }
for i in range(len(card_drivers_keys)):
	card_drivers[card_drivers_keys[i]] = card_drivers_values[i]

arch_driver_map = { "alpha"   : ( "ati", "glint", "mga", "nv", "rendition",
                                  "s3virge", "siliconmotion", "tdfx", "tga",
                                  "vga" ),
                    "arm"     : ( "vga" ),
                    "i386"    : ( "apm", "ark", "ati", "chips", "cirrus",
                                  "cyrix", "fbdev", "glide", "glint", "i128",
                                  "i740", "i810", "imstt", "mga", "neomagic",
                                  "nv", "rendition", "s3virge", "savage",
                                  "siliconmotion", "sis", "tdfx", "tga",
                                  "trident", "tseng", "vesa", "vga" ),
                    "m68k"    : ( "ati", "chips", "fbdev", "glint", "mga",
                                  "s3virge", "sis", "trident", "vga" ),
                    "powerpc" : ( "ati", "chips", "fbdev", "glint", "mga",
                                  "s3virge", "sis", "trident", "vga" ),
                    "sparc"   : ( "fbdev", "glint", "sunbw2", "suncg14",
                                   "suncg3", "suncg6", "sunffb", "sunleo", "suntcx" ) }

screen_size_freqs_keys = [ "Up to 14 inches (355 mm)",
                           "15 inches (380 mm)",
                           "17 inches (430 mm)",
                           "19-20 inches (480-510 mm)",
                           "21 inches (530 mm) or more" ]

screen_size_freqs_values = [ "28-33 43-72",
                             "28-50 43-75",
                             "30-70 50-160",
                             "30-100 50-160",
                             "30-130 50-160" ]

screen_size_freqs = { }
for i in range(len(screen_size_freqs_keys)):
	screen_size_freqs[screen_size_freqs_keys[i]] = screen_size_freqs_values[i]

mode_list_freqs_keys = [ "640x480 @ 60Hz",
                         "640x480 @ 72Hz",
                         "800x600 @ 60Hz",
                         "800x600 @ 72Hz",
                         "800x600 @ 85Hz",
                         "1024x768 @ 60Hz",
                         "1024x768 @ 70Hz",
                         "1024x768 @ 75Hz",
                         "1152x864 @ 75Hz",
                         "1280x960 @ 60Hz",
                         "1280x960 @ 85Hz",
                         "1600x1200 @ 60Hz",
                         "1600x1200 @ 75Hz",
                         "1600x1200 @ 85Hz",
                         "1792x1344 @ 60Hz",
                         "1792x1344 @ 75Hz",
                         "1856x1392 @ 60Hz",
                         "1856x1392 @ 75Hz",
                         "1920x1440 @ 60Hz",
                         "1920x1440 @ 75Hz" ]

mode_list_freqs_values = [ "28-33 43-72",
                           "28-38 43-72",
                           "28-38 43-72",
                           "28-48 43-72",
                           "30-54 50-85",
                           "28-49 43-72",
                           "30-57 43-72",
                           "30-60 50-75",
                           "30-68 50-85",
                           "30-60 50-75",
                           "30-92 50-85",
                           "30-75 50-85",
                           "30-94 50-75",
                           "30-107 50-85",
                           "30-84 50-75",
                           "30-107 50-85",
                           "30-87 50-75",
                           "30-113 50-75",
                           "30-90 50-75",
                           "30-130 60-160" ]

mode_list_freqs = { }
for i in range(len(mode_list_freqs_keys)):
	mode_list_freqs[mode_list_freqs_keys[i]] = mode_list_freqs_values[i]

mouse_protocols = [ "Auto",
                    "BusMouse",
                    "ExplorerPS/2",
                    "GlidePoint",
                    "GlidePointPS/2",
                    "ImPS/2",
                    "IntelliMouse",
                    "Logitech",
                    "MMHitTab",
                    "MMSeries",
                    "Microsoft",
                    "MouseMan",
                    "MouseManPlusPS/2",
                    "MouseSystems",
                    "NetMousePS/2",
                    "NetScrollPS/2",
                    "PS/2",
                    "ThinkingMouse",
                    "ThinkingMousePS/2" ]

allowed_users_choices_keys = [ "Root Only",
                               "Console Users Only",
                               "Anybody" ]

allowed_users_choices_values = [ "rootonly",
                                 "console",
                                 "anybody" ]

xservers = [ "xserver-8514",
             "xserver-agx",
             "xserver-mach32",
             "xserver-mach8",
             "xserver-p9000",
             "xserver-s3",
             "xserver-svga",
             "xserver-xfree86" ]

allowed_users_choices = { }
for i in range(len(allowed_users_choices_keys)):
	allowed_users_choices[allowed_users_choices_keys[i]] = allowed_users_choices_values[i]

def get_optionmenu_index(optionmenu):
	menu = optionmenu.get_menu()
	children = menu.children()
	item = menu.get_active()
	for i in range(len(children)):
		if children[i] == item:
			break
	return i

# the name of this function is a misnomer; it does all widget sensitivity handling for 3.x vs. 4.x issues
def audit_device_driver_state(menushell, selected_xserver):
	if selected_xserver == "xserver-xfree86":
		my_wtree.get_widget("device_driver").set_sensitive(TRUE)
		my_wtree.get_widget("default_depth").set_sensitive(TRUE)
		my_wtree.get_widget("default_bpp").set_sensitive(FALSE)
	elif selected_xserver == "xserver-8514"   or \
	     selected_xserver == "xserver-agx"    or \
	     selected_xserver == "xserver-mach32" or \
	     selected_xserver == "xserver-mach8"  or \
	     selected_xserver == "xserver-p9000"  or \
	     selected_xserver == "xserver-s3"     or \
	     selected_xserver == "xserver-svga":
		my_wtree.get_widget("device_driver").set_sensitive(FALSE)
		my_wtree.get_widget("default_depth").set_sensitive(FALSE)
		my_wtree.get_widget("default_bpp").set_sensitive(TRUE)
	else:
		raise RuntimeError, "impossible value of default_xserver widget"

def audit_simple_button_state(*args):
	if my_wtree.get_widget("monitor_lcd").get_active():
		my_wtree.get_widget("simple_button").set_sensitive(FALSE)
	else:
		my_wtree.get_widget("simple_button").set_sensitive(TRUE)

def set_freqs(horiz_sync, vert_refresh):
	global current_horiz_sync, current_vert_refresh
	current_horiz_sync = horiz_sync
	current_vert_refresh = vert_refresh
	my_wtree.get_widget("horiz_sync_label").set_text("Horizontal Sync: " + horiz_sync + " kHz")
	my_wtree.get_widget("vert_refresh_label").set_text("Vertical Refresh: " + vert_refresh + " Hz")
	my_wtree.get_widget("horiz_sync_entry").set_text(horiz_sync)
	my_wtree.get_widget("vert_refresh_entry").set_text(vert_refresh)

def raise_simple_dialog(*args):
	my_wtree.get_widget("simple_dialog").show()

def raise_medium_dialog(*args):
	my_wtree.get_widget("medium_dialog").show()

def raise_expert_dialog(*args):
	my_wtree.get_widget("expert_dialog").show()

def freq_ok(freq):
	# returns 0 if input invalid, 1 if valid
	if freq == "" or re.search(r'[^0-9,. -]', freq):
		return 0
	else:
		return 1

def dialog_ok(*args):
	global current_horiz_sync, current_vert_refresh
	this_dialog = args[-1]
	if this_dialog == "simple":
		i = get_optionmenu_index(my_wtree.get_widget("screen_size"))
		(horiz_sync, vert_refresh) = re.split(r'\s+', screen_size_freqs_values[i], 1)
	elif this_dialog == "medium":
		i = get_optionmenu_index(my_wtree.get_widget("mode_list"))
		(horiz_sync, vert_refresh) = re.split(r'\s+', mode_list_freqs_values[i], 1)
	elif this_dialog == "expert":
		horiz_sync = my_wtree.get_widget("horiz_sync_entry").get_text()
		vert_refresh = my_wtree.get_widget("vert_refresh_entry").get_text()
		if (not freq_ok(horiz_sync)) or (not freq_ok(vert_refresh)):
			dialog = GnomeMessageBox("Invalid characters in frequency specification.  If you\ncontinue, the information you have entered will be ignored.\n\nContinue?", \
						 "warning", STOCK_BUTTON_YES, STOCK_BUTTON_NO)
			dialog.set_position(WIN_POS_CENTER)
			discard_garbage = dialog.run_and_close()
			if discard_garbage == 0: # yes, close the dialog anyway, ignoring the bogus input
				horiz_sync = current_horiz_sync
				vert_refresh = current_vert_refresh
			elif discard_garbage == 1: # no, don't close the dialog; fix the bad input
				return
			else: # user closed dialog with window manager
				return
	set_freqs(horiz_sync, vert_refresh)
	this_widget = my_wtree.get_widget(this_dialog + "_dialog")
	this_widget.hide()

def dialog_cancel(*args):
	this_dialog = args[-1]
	this_widget = my_wtree.get_widget(this_dialog + "_dialog")
	this_widget.hide()

def raise_32bits_dialog(*args):
	dialog = GnomeMessageBox("32-bit color is actually 24 bits of color information plus\n8 bits of alpha channel or simple zero padding; the X Window\nSystem can handle both kinds of data; select the 24 bit\nsetting if you need either alpha channel or zero padding.", "info", STOCK_BUTTON_OK)
	dialog.set_position(WIN_POS_CENTER)
	clue_them_in = dialog.run_and_close()

class Dexlet(configlet.Configlet):
	__displayname__ = "X configuration"
	__pagenames__ = [ "videocard_hbox_main",
			  "monitor_hbox_main",
			  "desktop_hbox_main",
			  "pointer_hbox_main",
			  "xserver_hbox_main" ]
	__pagedisplay__ = { "videocard_hbox_main" : "Video Card",
			    "monitor_hbox_main" : "Monitor",
			    "pointer_hbox_main" : "Pointer",
			    "desktop_hbox_main" : "Desktop",
			    "xserver_hbox_main" : "X server" }
	__description__ = "Select this option to configure video and input hardware to work with the X Window System."

	def append_debconf(self, name, value):
		self.debconf_data.append(name + ' ' + name + ' ' + value)

	def setup(self):
		# here is where geometry fixups can happen if need be

		# give our callback functions above access to the widget tree
		global my_wtree

		my_wtree = self.wtree

		# connect the callback functions
		my_wtree.get_widget("simple_button").connect("clicked", raise_simple_dialog)
		my_wtree.get_widget("medium_button").connect("clicked", raise_medium_dialog)
		my_wtree.get_widget("expert_button").connect("clicked", raise_expert_dialog)

		my_wtree.get_widget("simple_ok_button").connect("clicked", dialog_ok, "simple")
		my_wtree.get_widget("medium_ok_button").connect("clicked", dialog_ok, "medium")
		my_wtree.get_widget("expert_ok_button").connect("clicked", dialog_ok, "expert")
		my_wtree.get_widget("simple_cancel_button").connect("clicked", dialog_cancel, "simple")
		my_wtree.get_widget("medium_cancel_button").connect("clicked", dialog_cancel, "medium")
		my_wtree.get_widget("expert_cancel_button").connect("clicked", dialog_cancel, "expert")

		my_wtree.get_widget("about_32bits_button").connect("clicked", raise_32bits_dialog)

		my_wtree.get_widget("monitor_lcd").connect("toggled", audit_simple_button_state)

		# initialize the option menus

		# color depth
		newmenu = GtkMenu()
		for item in color_depths_keys:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("default_depth").set_menu(newmenu)

		# bpp
		newmenu = GtkMenu()
		for item in bpp_choices:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("default_bpp").set_menu(newmenu)

		# X server driver list
		# TODO: populate this using arch_driver_map
		newmenu = GtkMenu()
		for item in card_drivers_keys:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("device_driver").set_menu(newmenu)

		# mouse protocols
		#mouse_protocols.sort()
		newmenu = GtkMenu()
		for item in mouse_protocols:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("mouse_protocol").set_menu(newmenu)

		# default X server
		dpkg_output = os.popen(r"dpkg --get-selections | awk '/xserver-.*install$/{print $1}' | grep -v common")
		temp_list = dpkg_output.readlines()
		#temp_list.sort()
		server_list = map(string.strip, temp_list)
		newmenu = GtkMenu()
		for item in server_list:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			menuitem.connect("activate", audit_device_driver_state, item)
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("default_xserver").set_menu(newmenu)

		# allowed users
		newmenu = GtkMenu()
		for item in allowed_users_choices_keys:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("allowed_users").set_menu(newmenu)

		# screen size
		newmenu = GtkMenu()
		for item in screen_size_freqs_keys:
			label = GtkLabel(item)
			label.show()
			menuitem = GtkMenuItem()
			menuitem.add(label)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("screen_size").set_menu(newmenu)

		# mode list
		newmenu = GtkMenu()
		for item in mode_list_freqs_keys:
			menuitem = GtkMenuItem(item)
			menuitem.show()
			newmenu.append(menuitem)
		newmenu.show()
		my_wtree.get_widget("mode_list").set_menu(newmenu)

	def load_debconf(self, data):
		"set configlet widget state using debconf database values"
		global current_horiz_sync, current_vert_refresh, current_video_ram
		# have to get current default X server first
		# assume 4.x unless we're told otherwise
		configspace = "xserver-xfree86"
		default_xserver = "xserver-xfree86"
		for template in data:
			(name, question, value) = re.split(r'\s+', template, 2)
			if name == "shared/default-x-server":
				if value == "xserver-xfree86" or value == '""':
					my_wtree.get_widget("device_driver").set_sensitive(TRUE)
					my_wtree.get_widget("default_depth").set_sensitive(TRUE)
					my_wtree.get_widget("default_bpp").set_sensitive(FALSE)
				elif value == "xserver-8514"   or \
				     value == "xserver-agx"    or \
				     value == "xserver-mach32" or \
				     value == "xserver-mach8"  or \
				     value == "xserver-p9000"  or \
				     value == "xserver-s3"     or \
				     value == "xserver-svga":
					my_wtree.get_widget("device_driver").set_sensitive(FALSE)
					my_wtree.get_widget("default_depth").set_sensitive(FALSE)
					my_wtree.get_widget("default_bpp").set_sensitive(TRUE)
					configspace = "shared/xfree86v3"
					default_xserver = value
				else:
					raise RuntimeError, "implausible value of shared/default-x-server"

		#print "default-x-server value from debconf is", default_xserver
		i = xservers.index(default_xserver)
		#print "found it in our list at index", i
		self.wtree.get_widget("default_xserver").set_history(i)

		# set widgets according to corresponding debconf db info
		for template in data:
			(name, question, value) = re.split(r'\s+', template, 2)
			if value == '""':
				value = ""

			# available modes
			if name == (configspace + "/config/display/modes"):
				#print "debconf config display modes are", value
				if value == "":
					#print "was null, turning on all 4:3 modes"
					value = "1920x1440, 1856x1392, 1792x1344, 1600x1200, 1280x960, 1152x864, 1024x768, 800x600, 640x480"
				modelist = re.split(r',\s*', value)
				for mode in ("1920x1440", "1856x1392", "1792x1344", "1600x1200", "1280x1024", "1280x960", "1152x864", "1024x768", "800x600", "640x480"):
					if mode in modelist:
						#print "mode_", mode, "widget ON"
						self.wtree.get_widget("mode_" + mode).set_active(TRUE)
					else:
						#print "mode_", mode, "widget OFF"
						self.wtree.get_widget("mode_" + mode).set_active(FALSE)

			# color depth
			elif name == (configspace + "/config/display/default_depth"):
				if value == "":
					value = "24"
				# value will be 1, 4, 8, 15, 16, or 24
				#print "default_depth value from debconf is", value
				i = color_depths_values.index(value)
				#print "found it in our list at index", i, "-- now setting default_depth widget to that"
				self.wtree.get_widget("default_depth").set_history(i)

			# bpp
			elif name == (configspace + "/config/display/default_bpp"):
				if value == "":
					value = "24"
				# value will be 8, 16, 24, or 32
				#print "default_bpp value from debconf is", value
				i = bpp_choices.index(value)
				#print "found it in our list at index", i, "-- now setting default_bpp widget to that"
				self.wtree.get_widget("default_bpp").set_history(i)

			# video card identifier
			elif name == (configspace + "/config/device/identifier"):
				if value == "":
					value = "Generic Video Card"
				self.wtree.get_widget("device_identifier").set_text(value)

			# X server driver (set inactive if 3.x)
			elif name == (configspace + "/config/device/driver"):
				if value == "":
					value = "ati"
				#print "device driver value from debconf is", value
				i = card_drivers_values.index(value)
				#print "found it in our list at index", i
				self.wtree.get_widget("device_driver").set_history(i)

			# video ram
			elif name == (configspace + "/config/device/video_ram"):
				self.wtree.get_widget("device_video_ram").set_text(value)
				current_video_ram = value

			# monitor identifier
			elif name == (configspace + "/config/monitor/identifier"):
				if value == "":
					value = "Generic Monitor"
				self.wtree.get_widget("monitor_identifier").set_text(value)

			# monitor is LCD
			elif name == (configspace + "/config/monitor/lcd"):
				if value == "":
					value = "false"
				self.wtree.get_widget("monitor_lcd").set_active(string.upper(value) == "TRUE")

			# screen size
			elif name == (configspace + "/config/monitor/screen-size"):
				if value == "":
					value = "17 inches (430 mm)"
				#print "monitor screen size value from debconf is", value
				i = screen_size_freqs_keys.index(value)
				#print "found it in our list at index", i
				self.wtree.get_widget("screen_size").set_history(i)

			# mode list
			elif name == (configspace + "/config/monitor/mode-list"):
				if value == "":
					value = "1152x864 @ 75Hz"
				#print "monitor mode list value from debconf is", value
				i = mode_list_freqs_keys.index(value)
				#print "found it in our list at index", i
				self.wtree.get_widget("mode_list").set_history(i)

			# horizontal sync
			elif name == (configspace + "/config/monitor/horiz-sync"):
				if value == "":
					value = "30-75"
				self.wtree.get_widget("horiz_sync_label").set_text("Horizontal Sync: " + value + " kHz")
				self.wtree.get_widget("horiz_sync_entry").set_text(value)
				current_horiz_sync = value

			# vertical refresh
			elif name == (configspace + "/config/monitor/vert-refresh"):
				if value == "":
					value = "50-85"
				self.wtree.get_widget("vert_refresh_label").set_text("Vertical Refresh: " + value + " Hz")
				self.wtree.get_widget("vert_refresh_entry").set_text(value)
				current_vert_refresh = value

			# mouse port
			elif name == (configspace + "/config/inputdevice/mouse/port"):
				if value == "":
					value = "/dev/psaux"
				self.wtree.get_widget("mouse_port").set_text(value)

			# mouse protocol
			# TODO: change available protocols based on mouse port
			elif name == (configspace + "/config/inputdevice/mouse/protocol"):
				if value == "":
					value = "PS/2"
				#print "mouse protocol from debconf is", value
				i = mouse_protocols.index(value)
				#print "found it in our list at", i
				self.wtree.get_widget("mouse_protocol").set_history(i)

			# emulate 3 buttons
			elif name == (configspace + "/config/inputdevice/mouse/emulate3buttons"):
				if value == "":
					value = "true"
				self.wtree.get_widget("emulate_3buttons").set_active(string.upper(value) == "TRUE")

			# scroll events
			elif name == (configspace + "/config/inputdevice/mouse/zaxismapping"):
				if value == "":
					value = "true"
				self.wtree.get_widget("enable_scroll_wheel").set_active(string.upper(value) == "TRUE")

			# X server
			# already handled above
			elif name == "shared/default-x-server":
				pass

			# allowed users
			elif name == "xserver-common/xwrapper/actual_allowed_users":
				if value == "":
					value = "console"
				#print "actual allowed users from debconf is", value
				i = allowed_users_choices_values.index(value)
				#print "found it in our list at index", i
				self.wtree.get_widget("allowed_users").set_history(i)

			# nice value
			elif name == "xserver-common/xwrapper/nice_value":
				if value == "":
					value = "-10"
				self.wtree.get_widget("nice_value").set_text(value)

			else:
				pass

	def validate_page(self, pagename):
		global current_video_ram
		if pagename == "videocard_hbox_main":
			video_ram_entry = my_wtree.get_widget("device_video_ram")
			if video_ram_entry.get_text() != "" and re.search(r'[^0-9]', video_ram_entry.get_text()):
				dialog = GnomeMessageBox("Invalid characters in video RAM specification.\nRestoring to previous value.", \
							 "warning", STOCK_BUTTON_OK)
				dialog.set_position(WIN_POS_CENTER)
				discard_garbage = dialog.run_and_close()
				video_ram_entry.set_text(current_video_ram)
		return 1

	def on_close(self):
		"collect and store configlet widget states"
		# default X server
		i = get_optionmenu_index(self.wtree.get_widget("default_xserver"))
		default_xserver = xservers[i]
		if default_xserver == "xserver-xfree86":
			configspace = default_xserver
		else:
			configspace = "shared/xfree86v3"

		# ACTUAL allowed users
		i = get_optionmenu_index(self.wtree.get_widget("allowed_users"))
		allowed_users = allowed_users_choices_keys[i]
		actual_allowed_users = allowed_users_choices_values[i]

		# nice value
		nice_value = self.wtree.get_widget("nice_value").get_text()

		# available modes
		temp = [ ]
		for mode in ("1920x1440", "1856x1392", "1792x1344", "1600x1200", "1280x1024", "1280x960", "1152x864", "1024x768", "800x600", "640x480"):
			if self.wtree.get_widget("mode_" + mode).get_active() == TRUE:
				temp.append(mode)
		modelist = string.join(temp, ", ")

		# color depth
		i = get_optionmenu_index(self.wtree.get_widget("default_depth"))
		color_depth = color_depths_values[i]

		# video card identifier
		device_identifier = string.strip(self.wtree.get_widget("device_identifier").get_text())
		# vendor and model are only distinct for version 3.x servers
		if default_xserver != "xserver-xfree86":
			if re.match(r'\s+', device_identifier):
				(device_vendor, device_model) = re.split(r'\s+', device_identifier, 1)
			else:
				device_vendor = "Generic"
				device_model = device_identifier

		# X server driver (set inactive if 3.x)
		if default_xserver == "xserver-xfree86":
			i = get_optionmenu_index(self.wtree.get_widget("device_driver"))
			xserver_driver = card_drivers_values[i]

		# video ram
		video_ram = string.strip(self.wtree.get_widget("device_video_ram").get_text())

		# monitor identifier
		monitor_identifier = string.strip(self.wtree.get_widget("monitor_identifier").get_text())
		# vendor and model are only distinct for version 3.x servers
		if default_xserver != "xserver-xfree86":
			if re.match(r'\s+', monitor_identifier):
				(monitor_vendor, monitor_model) = re.split(r'\s+', monitor_identifier, 1)
			else:
				monitor_vendor = "Generic"
				monitor_model = monitor_identifier

		# monitor is LCD
		if self.wtree.get_widget("monitor_lcd").get_active() == TRUE:
			monitor_lcd = "true"
		else:
			monitor_lcd = "false"

		# screen size
		i = get_optionmenu_index(self.wtree.get_widget("screen_size"))
		screen_size = screen_size_freqs_keys[i]

		# mode list
		i = get_optionmenu_index(self.wtree.get_widget("mode_list"))
		monitor_mode_list = mode_list_freqs_keys[i]

		# horizontal sync
		horiz_sync = current_horiz_sync

		# vertical refresh
		vert_refresh = current_vert_refresh

		# mouse port
		mouse_port = string.strip(self.wtree.get_widget("mouse_port").get_text())

		# mouse protocol
		i = get_optionmenu_index(self.wtree.get_widget("mouse_protocol"))
		mouse_protocol = mouse_protocols[i]

		# emulate 3 buttons
		if self.wtree.get_widget("emulate_3buttons").get_active() == TRUE:
			emulate_3buttons = "true"
		else:
			emulate_3buttons = "false"

		# scroll events
		if self.wtree.get_widget("enable_scroll_wheel").get_active() == TRUE:
			zaxismapping = "true"
		else:
			zaxismapping = "false"

		self.debconf_data = [ ]

		self.append_debconf("shared/default-x-server", default_xserver)
		self.append_debconf("xserver-common/xwrapper/allowed_users", allowed_users)
		self.append_debconf("xserver-common/xwrapper/actual_allowed_users", actual_allowed_users)
		self.append_debconf("xserver-common/xwrapper/nice_value", nice_value)
		self.append_debconf(configspace + "/config/device/identifier", device_identifier)
		self.append_debconf(configspace + "/config/device/video_ram", video_ram)
		self.append_debconf(configspace + "/config/inputdevice/mouse/port", mouse_port)
		self.append_debconf(configspace + "/config/inputdevice/mouse/protocol", mouse_protocol)
		self.append_debconf(configspace + "/config/inputdevice/mouse/emulate3buttons", emulate_3buttons)
		self.append_debconf(configspace + "/config/inputdevice/mouse/zaxismapping", zaxismapping)
		self.append_debconf(configspace + "/config/monitor/lcd", monitor_lcd)
		self.append_debconf(configspace + "/config/monitor/screen-size", screen_size)
		self.append_debconf(configspace + "/config/monitor/mode-list", monitor_mode_list)
		self.append_debconf(configspace + "/config/monitor/identifier", monitor_identifier)
		self.append_debconf(configspace + "/config/monitor/horiz-sync", horiz_sync)
		self.append_debconf(configspace + "/config/monitor/vert-refresh", vert_refresh)
		self.append_debconf(configspace + "/config/display/modes", modelist)
		self.append_debconf(configspace + "/config/display/default_depth", color_depth)

		if default_xserver == "xserver-xfree86":
			# if 4.x, append xserver_driver
			self.append_debconf(configspace + "/config/device/driver", xserver_driver)
		else:
			# if 3.x, append {device,monitor}_{vendor,model}
			self.append_debconf(configspace + "/config/device/vendor", device_vendor)
			self.append_debconf(configspace + "/config/device/model", device_model)
			self.append_debconf(configspace + "/config/monitor/vendor", monitor_vendor)
			self.append_debconf(configspace + "/config/monitor/model", monitor_model)

		# now set the questions that will cause the changes to take effect in the real config files
		self.append_debconf("xserver-common/clobber_xwrapper_config", "true")
		self.append_debconf("shared/clobber_x-server_symlink", "true")
		if default_xserver == "xserver-xfree86":
			self.append_debconf(configspace + "/clobber_XF86Config-4", "true")
		else:
			self.append_debconf(configspace + "/clobber_XF86Config", "true")

	def debconf(self):
		return self.debconf_data

configlet.register_configlet(Dexlet)

# vim:set ai noet softtabstop=8 sw=8 tw=0:
